@@ -3,7 +3,7 @@
 #
 #The MIT License (MIT)
 #
-#Copyright (c) <2013> <Colin Duquesnoy and others, see AUTHORS.txt>
+#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
 #
 #Permission is hereby granted, free of charge, to any person obtaining a copy
 #of this software and associated documentation files (the "Software"), to deal
@@ -27,13 +27,15 @@
 Contains the JediCompletionProvider class implementation.
 """
 import os
-import jedi
 from pyqode.core import Mode, DelayJobRunner, logger, constants
-from pyqode.core import CodeCompletionMode
+from pyqode.core import get_server
+from pyqode.core import Worker
 from pyqode.qt import QtCore, QtGui
 
 
-class CalltipsWorker(object):
+class CalltipsWorker(Worker):
+    _slot = "jedi"
+
     def __init__(self, code, line, col, path, encoding):
         self.code = code
         self.line = line
@@ -42,13 +44,14 @@ class CalltipsWorker(object):
         self.encoding = encoding
 
     def __call__(self, *args, **kwargs):
+        import jedi
         script = jedi.Script(self.code, self.line, self.col, self.path,
                              self.encoding)
         signatures = script.call_signatures()
         for c in signatures:
             results = [str(c.module.name), str(c.call_name),
                        [str(p.token_list[0]) for p in c.params], c.index,
-                       c.bracket_start]
+                       c.bracket_start, self.col]
             # seems like len of signatures is always 1 when getting calltips
             return results
         return []
@@ -80,10 +83,11 @@ class CalltipsMode(Mode, QtCore.QObject):
             if state:
                 self.editor.keyReleased.connect(self.__onKeyReleased)
 
-                CodeCompletionMode.SERVER.signals.workCompleted.connect(
-                    self.__onWorkFinished)
-            else:
-                CodeCompletionMode.SERVER.signals.workCompleted.disconnect(
+                if get_server():
+                    get_server().signals.workCompleted.connect(
+                        self.__onWorkFinished)
+            elif get_server():
+                get_server().signals.workCompleted.disconnect(
                     self.__onWorkFinished)
 
     def __onKeyReleased(self, event):
@@ -95,6 +99,13 @@ class CalltipsMode(Mode, QtCore.QObject):
             fn = self.editor.filePath
             encoding = self.editor.fileEncoding
             source = self.editor.toPlainText()
+            # jedi has a bug if the statement has a closing parenthesis
+            # remove it!
+            lines = source.splitlines()
+            l = lines[line - 1].rstrip()
+            if l.endswith(")"):
+                lines[line - 1] = l[:-1]
+            source = "\n".join(lines)
             self.__requestCalltip(source, line, col, fn, encoding)
 
     def __requestCalltip(self, *args):
@@ -102,7 +113,7 @@ class CalltipsMode(Mode, QtCore.QObject):
             self.__requestCnt += 1
             logger.debug("Calltip requested")
             worker = CalltipsWorker(*args)
-            CodeCompletionMode.SERVER.requestWork(self, worker)
+            get_server().requestWork(self, worker)
 
     def __onWorkFinished(self, caller_id, worker, results):
         if caller_id == id(self) and isinstance(worker, CalltipsWorker):
@@ -114,7 +125,7 @@ class CalltipsMode(Mode, QtCore.QObject):
                         "call.params": results[2],
                         "call.index": results[3],
                         "call.bracket_start": results[4]}
-                self.tooltipDisplayRequested.emit(call, worker.col)
+                self.tooltipDisplayRequested.emit(call, results[5])
 
     def __isLastCharEndOfWord(self):
         try:
@@ -149,7 +160,7 @@ class CalltipsMode(Mode, QtCore.QObject):
         w_offset = (col - call['call.bracket_start'][1]) * charWidth
         position = QtCore.QPoint(
             self.editor.cursorRect().x() - w_offset,
-            self.editor.cursorRect().y())
+            self.editor.cursorRect().y() + 35)
         position = self.editor.mapToGlobal(position)
         # show tooltip
         QtGui.QToolTip.showText(position, calltip, self.editor)
